/* Copyright 2021-2022 NVIDIA Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

#pragma once

#include "cunumeric/cunumeric.h"

namespace cunumeric {

// Match these to BitGeneratorOperation in config.py
enum class BitGeneratorOperation : int32_t {
  CREATE       = CUNUMERIC_BITGENOP_CREATE,
  DESTROY      = CUNUMERIC_BITGENOP_DESTROY,
  RAND_RAW     = CUNUMERIC_BITGENOP_RAND_RAW,
  DISTRIBUTION = CUNUMERIC_BITGENOP_DISTRIBUTION,
};

// Match these to BitGeneratorType in config.py
enum class BitGeneratorType : uint32_t {
  DEFAULT       = CUNUMERIC_BITGENTYPE_DEFAULT,
  XORWOW        = CUNUMERIC_BITGENTYPE_XORWOW,
  MRG32K3A      = CUNUMERIC_BITGENTYPE_MRG32K3A,
  MTGP32        = CUNUMERIC_BITGENTYPE_MTGP32,
  MT19937       = CUNUMERIC_BITGENTYPE_MT19937,
  PHILOX4_32_10 = CUNUMERIC_BITGENTYPE_PHILOX4_32_10,
};

// Match these to BitGeneratorDistribution in config.py
enum class BitGeneratorDistribution : int32_t {
  INTEGERS_16       = CUNUMERIC_BITGENDIST_INTEGERS_16,
  INTEGERS_32       = CUNUMERIC_BITGENDIST_INTEGERS_32,
  INTEGERS_64       = CUNUMERIC_BITGENDIST_INTEGERS_64,
  UNIFORM_32        = CUNUMERIC_BITGENDIST_UNIFORM_32,
  UNIFORM_64        = CUNUMERIC_BITGENDIST_UNIFORM_64,
  LOGNORMAL_32      = CUNUMERIC_BITGENDIST_LOGNORMAL_32,
  LOGNORMAL_64      = CUNUMERIC_BITGENDIST_LOGNORMAL_64,
  NORMAL_32         = CUNUMERIC_BITGENDIST_NORMAL_32,
  NORMAL_64         = CUNUMERIC_BITGENDIST_NORMAL_64,
  POISSON           = CUNUMERIC_BITGENDIST_POISSON,
  EXPONENTIAL_32    = CUNUMERIC_BITGENDIST_EXPONENTIAL_32,
  EXPONENTIAL_64    = CUNUMERIC_BITGENDIST_EXPONENTIAL_64,
  GUMBEL_32         = CUNUMERIC_BITGENDIST_GUMBEL_32,
  GUMBEL_64         = CUNUMERIC_BITGENDIST_GUMBEL_64,
  LAPLACE_32        = CUNUMERIC_BITGENDIST_LAPLACE_32,
  LAPLACE_64        = CUNUMERIC_BITGENDIST_LAPLACE_64,
  LOGISTIC_32       = CUNUMERIC_BITGENDIST_LOGISTIC_32,
  LOGISTIC_64       = CUNUMERIC_BITGENDIST_LOGISTIC_64,
  PARETO_32         = CUNUMERIC_BITGENDIST_PARETO_32,
  PARETO_64         = CUNUMERIC_BITGENDIST_PARETO_64,
  POWER_32          = CUNUMERIC_BITGENDIST_POWER_32,
  POWER_64          = CUNUMERIC_BITGENDIST_POWER_64,
  RAYLEIGH_32       = CUNUMERIC_BITGENDIST_RAYLEIGH_32,
  RAYLEIGH_64       = CUNUMERIC_BITGENDIST_RAYLEIGH_64,
  CAUCHY_32         = CUNUMERIC_BITGENDIST_CAUCHY_32,
  CAUCHY_64         = CUNUMERIC_BITGENDIST_CAUCHY_64,
  TRIANGULAR_32     = CUNUMERIC_BITGENDIST_TRIANGULAR_32,
  TRIANGULAR_64     = CUNUMERIC_BITGENDIST_TRIANGULAR_64,
  WEIBULL_32        = CUNUMERIC_BITGENDIST_WEIBULL_32,
  WEIBULL_64        = CUNUMERIC_BITGENDIST_WEIBULL_64,
  BYTES             = CUNUMERIC_BITGENDIST_BYTES,
  BETA_32           = CUNUMERIC_BITGENDIST_BETA_32,
  BETA_64           = CUNUMERIC_BITGENDIST_BETA_64,
  F_32              = CUNUMERIC_BITGENDIST_F_32,
  F_64              = CUNUMERIC_BITGENDIST_F_64,
  LOGSERIES         = CUNUMERIC_BITGENDIST_LOGSERIES,
  NONCENTRAL_F_32   = CUNUMERIC_BITGENDIST_NONCENTRAL_F_32,
  NONCENTRAL_F_64   = CUNUMERIC_BITGENDIST_NONCENTRAL_F_64,
  CHISQUARE_32      = CUNUMERIC_BITGENDIST_CHISQUARE_32,
  CHISQUARE_64      = CUNUMERIC_BITGENDIST_CHISQUARE_64,
  GAMMA_32          = CUNUMERIC_BITGENDIST_GAMMA_32,
  GAMMA_64          = CUNUMERIC_BITGENDIST_GAMMA_64,
  STANDARD_T_32     = CUNUMERIC_BITGENDIST_STANDARD_T_32,
  STANDARD_T_64     = CUNUMERIC_BITGENDIST_STANDARD_T_64,
  HYPERGEOMETRIC    = CUNUMERIC_BITGENDIST_HYPERGEOMETRIC,
  VONMISES_32       = CUNUMERIC_BITGENDIST_VONMISES_32,
  VONMISES_64       = CUNUMERIC_BITGENDIST_VONMISES_64,
  ZIPF              = CUNUMERIC_BITGENDIST_ZIPF,
  GEOMETRIC         = CUNUMERIC_BITGENDIST_GEOMETRIC,
  WALD_32           = CUNUMERIC_BITGENDIST_WALD_32,
  WALD_64           = CUNUMERIC_BITGENDIST_WALD_64,
  BINOMIAL          = CUNUMERIC_BITGENDIST_BINOMIAL,
  NEGATIVE_BINOMIAL = CUNUMERIC_BITGENDIST_NEGATIVE_BINOMIAL,
};

}  // namespace cunumeric
